--[[
This controls the configuration settings (and display of a sample set of
actions based on those settings) for each ActionSetLayout.  Each
BarActionSetLayout is associated with an BarActionSetLayoutConfig, and each
BarActionSetLayoutConfig refers to that same BarActionSetLayout instance.
--]]
uab.classes.OvalLayoutConfig = uab.mixins.aceOO.Class(IActionSetLayoutConfig, uab.mixins.aceEvent)
uab.classes.OvalLayoutConfig.count = 0

function uab.classes.OvalLayoutConfig.AnchorDisplayValue(value)
    if value == uab.constants.anchor.top then
        return "Top & Bottom"
    else
        return "Left & Right"
    end
end

function uab.classes.OvalLayoutConfig.StartLocationDisplayValue(value)
    if value == uab.constants.placement.top then
        return "Top"
    elseif value == uab.constants.placement.topright then
        return "Top Right"
    elseif value == uab.constants.placement.topleft then
        return "Top Left"
    elseif value == uab.constants.placement.bottom then
        return "Bottom"
    elseif value == uab.constants.placement.bottomright then
        return "Bottom Right"
    elseif value == uab.constants.placement.bottomleft then
        return "Bottom Left"
    elseif value == uab.constants.placement.left then
        return "Left"
    else
        return "Right"
    end
end

function uab.classes.OvalLayoutConfig.prototype:init(layout, settings)
    uab.classes.OvalLayoutConfig.super.prototype.init(self)

	self.frames = {}

	self.settings = settings

	self:SettingsChanged()

	uab.classes.OvalLayoutConfig.count = uab.classes.OvalLayoutConfig.count + 1
	self.number = uab.classes.OvalLayoutConfig.count
end

function uab.classes.OvalLayoutConfig.prototype:GetFrame(parent)
	if not self.frame then
	    self:CreateFrame(parent)
	end
	
	return self.frame
end

function uab.classes.OvalLayoutConfig.prototype:SettingsChanged(settings)
	if settings then
		self.settings = settings
	end

    self.settings.growClockwise = (settings and self.settings.growClockwise) or "true"
    self.settings.anchor = (settings and self.settings.anchor) or uab.constants.anchor.left
    self.settings.startLocation = (settings and self.settings.startLocation) or uab.constants.placement.topleft

    if self.frame then
		--[[
		Update the controls in the frame to show the new settings
		--]]
    	
    	self:SetGrowClockwise(self.settings.growClockwise)
	    self:SetStartLocation(self.settings.startLocation)
	    self:SetAnchor(self.settings.anchor)
	end
end

function uab.classes.OvalLayoutConfig.prototype:UseDefaults(bypassNotification)
	if not self.settings then
	    self.settings = {}
	end

    self.settings.growClockwise = "true"
    self.settings.anchor = uab.constants.anchor.left
    self.settings.startLocation = uab.constants.placement.topleft
    
	self:SettingsChanged()
	
    if not bypassNotification then
        uab.main:LayoutSettingsChanged()
    end
end

function uab.classes.OvalLayoutConfig.prototype:MenuOptionArgs()
	--[[
	Build an AceOptions table of the options to be used to configure this
	config instance.  This table is inserted into a parent menu as a set of
	options.  If there are several options, just define them at the top level
	of the returned table.  Do not create a 'group' at the top level to enclose
	the options.
	--]]
	
	local options = {
        clockwise = {
            type = 'toggle',
            name = "Grow clockwise",
            desc = "When buttons are added, add them in a clockwise direction",
            get = function()
                return self.settings.growClockwise == "true"
            end,
            set = function(v)
                if v then
					self:SetGrowClockwise("true")
				else
				    self:SetGrowClockwise("false")
                end
            end,
        },
        startLocation = {
            type = 'text',
            name = "First button location",
            desc = "Placement of the first button, relative to the cancel button",
            get = function()
                return uab.classes.OvalLayoutConfig.StartLocationDisplayValue(self.settings.startLocation)
            end,
            set = function(start)
                if start == uab.classes.OvalLayoutConfig.StartLocationDisplayValue(uab.constants.placement.topleft) then
                    self:SetStartLocation(uab.constants.placement.topleft)
                elseif start == uab.classes.OvalLayoutConfig.StartLocationDisplayValue(uab.constants.placement.top) then
                    self:SetStartLocation(uab.constants.placement.top)
                elseif start == uab.classes.OvalLayoutConfig.StartLocationDisplayValue(uab.constants.placement.topright) then
                    self:SetStartLocation(uab.constants.placement.topright)
				elseif start == uab.classes.OvalLayoutConfig.StartLocationDisplayValue(uab.constants.placement.right) then
                	self:SetStartLocation(uab.constants.placement.right)
				elseif start == uab.classes.OvalLayoutConfig.StartLocationDisplayValue(uab.constants.placement.bottomright) then
                	self:SetStartLocation(uab.constants.placement.bottomright)
				elseif start == uab.classes.OvalLayoutConfig.StartLocationDisplayValue(uab.constants.placement.bottom) then
                	self:SetStartLocation(uab.constants.placement.bottom)
				elseif start == uab.classes.OvalLayoutConfig.StartLocationDisplayValue(uab.constants.placement.bottomleft) then
                	self:SetStartLocation(uab.constants.placement.bottomleft)
				elseif start == uab.classes.OvalLayoutConfig.StartLocationDisplayValue(uab.constants.placement.left) then
                	self:SetStartLocation(uab.constants.placement.left)
				end
            end,

            validate = {
				uab.classes.OvalLayoutConfig.StartLocationDisplayValue(uab.constants.placement.topleft),
				uab.classes.OvalLayoutConfig.StartLocationDisplayValue(uab.constants.placement.top),
				uab.classes.OvalLayoutConfig.StartLocationDisplayValue(uab.constants.placement.topright),
				uab.classes.OvalLayoutConfig.StartLocationDisplayValue(uab.constants.placement.right),
				uab.classes.OvalLayoutConfig.StartLocationDisplayValue(uab.constants.placement.bottomright),
				uab.classes.OvalLayoutConfig.StartLocationDisplayValue(uab.constants.placement.bottom),
				uab.classes.OvalLayoutConfig.StartLocationDisplayValue(uab.constants.placement.bottomleft),
				uab.classes.OvalLayoutConfig.StartLocationDisplayValue(uab.constants.placement.left),
			},
        },
        anchor = {
            type = 'text',
            name = "Anchor location",
            desc = "Long side of the oval on the left/right or top/bottom",
            get = function()
                return uab.classes.OvalLayoutConfig.AnchorDisplayValue(self.settings.anchor)
            end,
            set = function(anchor)
                if anchor == uab.classes.OvalLayoutConfig.AnchorDisplayValue(uab.constants.anchor.left) then
	                self:SetAnchor(uab.constants.anchor.left)
				elseif anchor == uab.classes.OvalLayoutConfig.AnchorDisplayValue(uab.constants.anchor.top) then
	                self:SetAnchor(uab.constants.anchor.top)
				end
            end,

            validate = {
				uab.classes.OvalLayoutConfig.AnchorDisplayValue(uab.constants.anchor.left),
				uab.classes.OvalLayoutConfig.AnchorDisplayValue(uab.constants.anchor.top),
			},
        },
	}
	
	return options
end

--[[---------------------------------------------------------------------------
Helper methods, to be used only by the class itself
-----------------------------------------------------------------------------]]

function uab.classes.OvalLayoutConfig.prototype:CreateFrame(parent)
	self.frame = CreateFrame("Frame", "uabOvalOptions"..self.number, parent or UIParent, "UAB_OvalLayoutOptionsTemplate")

	self:SetupFrame()
end

function uab.classes.OvalLayoutConfig.prototype:SetupFrame()
	self.frames = {}

    self.frames.growDirection = getglobal(self.frame:GetName().."GrowDirection")
    self.frames.growDirection.config = self
	
    getglobal(self.frames.growDirection:GetName().."Text"):SetText("Add buttons clockwise")
	self.frames.growDirection:SetScript("OnClick", function(self, button, down) self.config:GrowDirectionButtonClicked(self) end)

    self.frames.startLocation = getglobal(self.frame:GetName().."StartLocation")
    self.frames.startLocation.config = self

    uab.classes.OvalLayoutConfig.InsideInit = self
    
    UIDropDownMenu_Initialize(self.frames.startLocation,
        function()
            if uab.classes.OvalLayoutConfig.InsideInit then
                uab.classes.OvalLayoutConfig.InsideInit:StartLocationListInitialize()
            else
                this:GetParent().config:StartLocationListInitialize()
            end
        end)
	UIDropDownMenu_SetWidth(110, self.frames.startLocation)
	UIDropDownMenu_SetButtonWidth(24, self.frames.startLocation)
	UIDropDownMenu_JustifyText("LEFT", self.frames.startLocation)

    self.frames.anchor = getglobal(self.frame:GetName().."Anchor")
    self.frames.anchor.config = self
	
    uab.classes.OvalLayoutConfig.InsideInit = self
    
    UIDropDownMenu_Initialize(self.frames.anchor,
        function()
            if uab.classes.OvalLayoutConfig.InsideInit then
                uab.classes.OvalLayoutConfig.InsideInit:AnchorListInitialize()
            else
                this:GetParent().config:AnchorListInitialize()
            end
        end)
	UIDropDownMenu_SetWidth(110, self.frames.anchor)
	UIDropDownMenu_SetButtonWidth(24, self.frames.anchor)
	UIDropDownMenu_JustifyText("LEFT", self.frames.anchor)

    uab.classes.OvalLayoutConfig.InsideInit = nil
    
	self:SetGrowClockwise(self.settings.growClockwise)
	self:SetStartLocation(self.settings.startLocation)
	self:SetAnchor(self.settings.anchor)
end

function uab.classes.OvalLayoutConfig.prototype:StartLocationListInitialize()
    local index = 1
    local selected = nil

    local options = {}

    table.insert(options, {	text = "Top Left",
                            func = uab.DropDownSelect,
                            arg1 = self,
                            arg2 = { func = "SelectStartLocation", arg1 = uab.constants.placement.topleft }})
    table.insert(options, {	text = "Top",
                            func = uab.DropDownSelect,
                            arg1 = self,
                            arg2 = { func = "SelectStartLocation", arg1 = uab.constants.placement.top }})
    table.insert(options, {	text = "Top Right",
                            func = uab.DropDownSelect,
                            arg1 = self,
                            arg2 = { func = "SelectStartLocation", arg1 = uab.constants.placement.topright }})
    table.insert(options, {	text = "Right",
                            func = uab.DropDownSelect,
                            arg1 = self,
                            arg2 = { func = "SelectStartLocation", arg1 = uab.constants.placement.right }})
    table.insert(options, {	text = "Bottom Right",
                            func = uab.DropDownSelect,
                            arg1 = self,
                            arg2 = { func = "SelectStartLocation", arg1 = uab.constants.placement.bottomright }})
    table.insert(options, {	text = "Bottom",
                            func = uab.DropDownSelect,
                            arg1 = self,
                            arg2 = { func = "SelectStartLocation", arg1 = uab.constants.placement.bottom }})
    table.insert(options, {	text = "Bottom Left",
                            func = uab.DropDownSelect,
                            arg1 = self,
                            arg2 = { func = "SelectStartLocation", arg1 = uab.constants.placement.bottomleft }})
    table.insert(options, {	text = "Left",
                            func = uab.DropDownSelect,
                            arg1 = self,
                            arg2 = { func = "SelectStartLocation", arg1 = uab.constants.placement.left }})

    for index, data in pairs(options) do
        UIDropDownMenu_AddButton(data)
        if self.settings.startLocation == data.arg2.arg1 then
            selected = index
        end
    end

    UIDropDownMenu_SetSelectedID(self.frames.startLocation, selected)
end

function uab.classes.OvalLayoutConfig.prototype:SelectStartLocation(selector, selection)
    UIDropDownMenu_SetSelectedID(self.frames.startLocation, selector:GetID())
    self:SetStartLocation(selection)
end

function uab.classes.OvalLayoutConfig.prototype:SetStartLocation(value)
    local notify = (value ~= self.settings.startLocation)
        
    self.settings.startLocation = value
    if self.settings.anchor == uab.constants.anchor.top then
        if self.settings.startLocation == uab.constants.placement.left or self.settings.startLocation == uab.constants.placement.right then
            self:SetAnchor(uab.constants.anchor.left)
        end
    elseif value == uab.constants.anchor.left then
        if self.settings.startLocation == uab.constants.placement.top or self.settings.startLocation == uab.constants.placement.bottom then
            self:SetAnchor(uab.constants.anchor.top)
        end
    end
    
    if self.frames.startLocation then
        UIDropDownMenu_SetText(uab.classes.OvalLayoutConfig.StartLocationDisplayValue(value), self.frames.startLocation)
    end
    
    if notify then
        uab.main:LayoutSettingsChanged()
    end        
end

function uab.classes.OvalLayoutConfig.prototype:AnchorListInitialize()
    local index = 1
    local selected = nil

    local anchorOptions = {}

    table.insert(anchorOptions, {	text = "Top & Bottom",
                                    func = uab.DropDownSelect,
                                    arg1 = self,
                                    arg2 = { func = "SelectAnchor", arg1 = uab.constants.anchor.top }})
    table.insert(anchorOptions, {	text = "Left & Right",
                                    func = uab.DropDownSelect,
                                    arg1 = self,
                                    arg2 = { func = "SelectAnchor", arg1 = uab.constants.anchor.left }})

    for index, data in pairs(anchorOptions) do
        UIDropDownMenu_AddButton(data)
        if self.settings.anchor == data.arg2.arg1 then
            selected = index
        end
    end

    UIDropDownMenu_SetSelectedID(self.frames.anchor, selected)
end

function uab.classes.OvalLayoutConfig.prototype:SelectAnchor(selector, selection)
    UIDropDownMenu_SetSelectedID(self.frames.anchor, selector:GetID())
    self:SetAnchor(selection)
end

function uab.classes.OvalLayoutConfig.prototype:SetAnchor(value)
    local notify = (value ~= self.settings.anchor)
    
    self.settings.anchor = value
    
    if self.frames.anchor then
        UIDropDownMenu_SetText(uab.classes.OvalLayoutConfig.AnchorDisplayValue(value), self.frames.anchor)
    end
    
    if self.settings.anchor == uab.constants.anchor.top then
        if self.settings.startLocation == uab.constants.placement.left or self.settings.startLocation == uab.constants.placement.right then
            self:SetStartLocation(uab.constants.placement.top..self.settings.startLocation)
        end
    elseif value == uab.constants.anchor.left then
        if self.settings.startLocation == uab.constants.placement.top or self.settings.startLocation == uab.constants.placement.bottom then
            self:SetStartLocation(self.settings.startLocation..uab.constants.placement.left)
        end
    end
    
    if notify then
        uab.main:LayoutSettingsChanged()
    end        
end

function uab.classes.OvalLayoutConfig.prototype:GrowDirectionButtonClicked(frame)
    self:SetGrowClockwise(frame:GetChecked())
end

function uab.classes.OvalLayoutConfig.prototype:SetGrowClockwise(value)
    local newval
    
    if value and (tonumber(value) == 1 or value == "true") then
        newval = "true"
    else
        newval = "false"
    end
    
    local notify = (newval ~= self.settings.growClockwise)
        
    self.settings.growClockwise = newval
    
    if self.frames.growClockwise then
        self.frames.growDirection:SetChecked(self.settings.growClockwise == "true")
    end
    
    if notify then
        uab.main:LayoutSettingsChanged()
    end        
end

